-- ******************************************************************************************* --
-- * Create Asset Catalogue Specification Hierarchies relating to BS ISO 14224 ISO (MXES)    * --
-- ******************************************************************************************* --
-- *                                                                                         * --
-- * Implements MRO interpretation of BS ISO 14224:2006 Asset specifications for Oil and Gas * --
-- *                                                                                         * --
-- * This version for MXES environment (7.1.0)                                               * --
-- *                                                                                         * --
-- * This process generates a number of DB2 Stored Procedures within the target	     		 * --
-- * schema, it then creates additional Units of Measure (MEASUREUNIT), ALN Value     	     * --
-- * lists (MAXDOMAIN and ALNDOMAIN), Asset Attributes (ASSETATTRIBUTE). It then generates   * --
-- * additional Asset Classification structures (CLASSIFICATION, CLASSSTRUCTURE,   			 * --
-- * CLASSUSEWITH, CLASSSPEC, CLASSSPECUSEWITH, CLASSANCESTOR) entries corresponding to the  * --
-- * ISO 14224 Standard.                         											 * --
-- *                                                                                         * --                                                                          
-- *                                                                                         * --
-- * This script is based on organisation and can be run many times for each organisation    * --
-- * requiring the data.                                                                     * --
-- *                                                                                         * --
-- * Revision History                                                                        * --
-- *											     										 * --
-- * Original implementation of Asset Catalogue Hierarchy Script.                            * --
-- *                                       (version 1.10, 3rd March 2008, Denis Rodrigo)  	 * --
-- ******************************************************************************************* --

-- ******************************************************************************************* --
-- * 1) Create Stored Procedures used by Load Process                           	     	 * --
-- ******************************************************************************************* --
/* PROCEDURE Insert_Measure_Unit_Record */
CREATE PROCEDURE PLUSG_AS_1 ( v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iMeasureUnitSeq INTEGER;

    DECLARE v_vMeasurementUnitExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vMeasureUnit VARCHAR(8);

    DECLARE v_vAbbreviation VARCHAR(8);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE v_vOnGPrefix VARCHAR(10);

    DECLARE cur_1 CURSOR FOR SELECT measureunitid, abbreviation, description
                             FROM plusg_as_aux1;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('MEASUREUNITID', 'ABBREVIATION', 'DESCRIPTION', 'MEASUREUNITUID', 'CONTENTUID')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vMeasurementUnitExists = 'N';

    SET v_vObjectName = 'MEASUREUNIT';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

    SET v_vOnGPrefix = 'BMXAO';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vMeasureUnit,
                              v_vAbbreviation,
                              v_vDesc;

        WHILE l_sqlstatus = 0 DO

            SET v_vMeasurementUnitExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM measureunit
                                        WHERE measureunitid = v_vMeasureUnit
                                        AND orgid = v_pOrgID);

            IF v_vMeasurementUnitExists = 'Y' THEN

                UPDATE measureunit
                SET description = v_vDesc, abbreviation = v_vAbbreviation
                WHERE measureunitid = v_vMeasureUnit
                AND orgid = v_pOrgID;

            ELSE

                VALUES NEXTVAL FOR MEASUREUNITSEQ INTO v_iMeasureUnitSeq;

                SET v_vSql_stmt = 'INSERT INTO measureunit (measureunitid, abbreviation, description, measureunituid, contentuid, ' || v_vColumns || ') VALUES (''' || v_vMeasureUnit || ''', ''' || v_vAbbreviation || ''', ''' || v_vDesc || ''', ' || REPLACE(CHAR(v_iMeasureUnitSeq), ' ', '') || ', ''' || v_vOnGPrefix || CHAR(v_iMeasureUnitSeq) || ''', ' || v_vValues || ')';

                EXECUTE IMMEDIATE v_vSql_stmt;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vMeasureUnit,
                                  v_vAbbreviation,
                                  v_vDesc;

        END WHILE;

        CLOSE cur_1;

END
GO

/* PROCEDURE Create_MaxDomain_Record */
CREATE PROCEDURE PLUSG_AS_2 ()
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vDomainID VARCHAR(18);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE v_vDomainType VARCHAR(20);

    DECLARE v_vMaxTypeDomain VARCHAR(8);

    DECLARE v_iLength INTEGER;

    DECLARE cur_1 CURSOR FOR SELECT domainid, description, domaintype, maxtype, length
                             FROM plusg_as_aux2
                             WHERE domainid NOT IN (SELECT domainid FROM maxdomain);

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND required = 1
                             AND attributename NOT IN ('DOMAINID', 'DESCRIPTION', 'DOMAINTYPE', 'MAXTYPE', 'LENGTH', 'MAXDOMAINID')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vObjectName = 'MAXDOMAIN';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vDomainID,
                              v_vDesc,
                              v_vDomainType,
                              v_vMaxTypeDomain,
                              v_iLength;

        WHILE l_sqlstatus = 0 DO

            SET v_vSql_stmt = 'INSERT INTO maxdomain (domainid, description, domaintype, maxtype, length, maxdomainid, ' || v_vColumns || ') VALUES (''' || v_vDomainID || ''', ''' || v_vDesc || ''', ''' || v_vDomainType || ''', ''' || v_vMaxTypeDomain || ''', ' || REPLACE(CHAR(v_iLength), ' ', '') || ', ' || REPLACE(CHAR(NEXTVAL FOR MAXDOMAINSEQ), ' ', '') || ', ' || v_vValues || ')';

            EXECUTE IMMEDIATE v_vSql_stmt;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vDomainID,
                                  v_vDesc,
                                  v_vDomainType,
                                  v_vMaxTypeDomain,
                                  v_iLength;

        END WHILE;

        CLOSE cur_1;

END 
GO

/* PROCEDURE Create_ALNDomain_Record */
CREATE PROCEDURE PLUSG_AS_3 ()
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iALNDomainSeq INTEGER;

    DECLARE v_vALNDomainExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vDomainID VARCHAR(18);

    DECLARE v_vValue VARCHAR(100);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE cur_1 CURSOR FOR SELECT domainid, value, description
                             FROM plusg_as_aux3;

    /*DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND required = 1
                             AND attributename NOT IN ('DOMAINID', 'VALUE', 'DESCRIPTION', 'ALNDOMAINID', 'VALUEID')
                             ORDER BY attributeno;*/

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vALNDomainExists = 'N';

    SET v_vObjectName = 'ALNDOMAIN';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        /*OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;*/

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vDomainID,
                              v_vValue,
                              v_vDesc;

        WHILE l_sqlstatus = 0 DO

            SET v_vALNDomainExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM alndomain
                                        WHERE UPPER(value) = UPPER(v_vValue)
                                        AND domainid = v_vDomainID);

            IF v_vALNDomainExists = 'N' THEN

                SET v_vSql_stmt = 'INSERT INTO alndomain (domainid, value, description, alndomainid, valueid) VALUES (''' || v_vDomainID || ''', ''' || v_vValue || ''', ''' || v_vDesc || ''', ' || REPLACE(CHAR(NEXTVAL FOR ALNDOMAINSEQ), ' ', '') || ', ''' || v_vDomainID || '|' || v_vValue || ''')';

                EXECUTE IMMEDIATE v_vSql_stmt;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vDomainID,
                                  v_vValue,
                                  v_vDesc;

        END WHILE;

        CLOSE cur_1;

END 
GO

/* PROCEDURE CreateAssetAttribute */
CREATE PROCEDURE PLUSG_AS_4 ( v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';
    
    DECLARE v_vAssetAttributeExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vAssetAttrID VARCHAR(8);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE v_vDataType VARCHAR(8);

    DECLARE v_vMeasureUnitID VARCHAR(8);

    DECLARE v_vDomainID VARCHAR(18);

    DECLARE cur_1 CURSOR FOR SELECT DISTINCT assetattrid, description, datatype, measureunitid, domainid
                             FROM plusg_as_aux4;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('ASSETATTRID', 'DESCRIPTION', 'DATATYPE', 'MEASUREUNITID', 'DOMAINID', 'ASSETATTRIBUTEID')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;
        
	SET v_vAssetAttributeExists = 'N';        

    SET v_vObjectName = 'ASSETATTRIBUTE';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vAssetAttrID,
                              v_vDesc,
                              v_vDataType,
                              v_vMeasureUnitID,
                              v_vDomainID;

        WHILE l_sqlstatus = 0 DO
        
            SET v_vAssetAttributeExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM assetattribute
                                        WHERE assetattrid = v_vAssetAttrID
                                        AND datatype = v_vDataType
                                        AND orgid = v_pOrgID);
                                        
            IF v_vAssetAttributeExists = 'Y' THEN
            
            	IF v_vDataType = 'NUMERIC' THEN

                	UPDATE assetattribute
                	SET description = v_vDesc, measureunitid = v_vMeasureUnitID
                	WHERE assetattrid = v_vAssetAttrID
                	AND orgid = v_pOrgID;
                	
                ELSE
                
                	UPDATE assetattribute
                	SET description = v_vDesc
                	WHERE assetattrid = v_vAssetAttrID
                	AND orgid = v_pOrgID;                
                	
                END IF;           
            
            ELSE                                       

            	SET v_vSql_stmt = 'INSERT INTO assetattribute (assetattrid, description, datatype, measureunitid, domainid, assetattributeid, ' || v_vColumns || ') VALUES (''' || v_vAssetAttrID || ''', ''' || v_vDesc || ''', ''' || v_vDataType || ''', ''' || v_vMeasureUnitID || ''', ''' || v_vDomainID || ''', ' || REPLACE(CHAR(NEXTVAL FOR ASSETATTRIBUTESEQ), ' ', '') || ', ' || v_vValues || ')';

            	EXECUTE IMMEDIATE v_vSql_stmt;
            	
			END IF;            	

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vAssetAttrID,
                                  v_vDesc,
                                  v_vDataType,
                                  v_vMeasureUnitID,
                                  v_vDomainID;

        END WHILE;

        CLOSE cur_1;

END
GO

/* PROCEDURE CreateClassification */
CREATE PROCEDURE PLUSG_AS_5 ( v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vClassificationExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vClassificationID VARCHAR(8);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE cur_1 CURSOR FOR SELECT classificationid, description
                             FROM plusg_as_aux5;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('CLASSIFICATIONID', 'DESCRIPTION', 'CLASSIFICATIONUID')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vClassificationExists = 'N';

    SET v_vObjectName = 'CLASSIFICATION';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vClassificationID,
                              v_vDesc;

        WHILE l_sqlstatus = 0 DO

            SET v_vClassificationExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM classification
                                        WHERE classificationid = v_vClassificationID
                                        AND orgid = v_pOrgID);

            IF v_vClassificationExists = 'Y' THEN

                UPDATE classification
                SET description = v_vDesc
                WHERE classificationid = v_vClassificationID
                AND orgid = v_pOrgID;

            ELSE

                SET v_vSql_stmt = 'INSERT INTO classification (classificationid, description, classificationuid, ' || v_vColumns || ') VALUES (''' || v_vClassificationID || ''', ''' || v_vDesc || ''', ' || REPLACE(CHAR(NEXTVAL FOR CLASSIFICATIONSEQ), ' ', '') || ', ' || v_vValues || ')';

                EXECUTE IMMEDIATE v_vSql_stmt;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vClassificationID,
                                  v_vDesc;

        END WHILE;

        CLOSE cur_1;

END 
GO

/* PROCEDURE CreateClassUseWith */
CREATE PROCEDURE PLUSG_AS_7 ( v_pClassStructureID VARCHAR(20) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vClassUseWithExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vValue VARCHAR(50);

    DECLARE v_vDesc VARCHAR(256);

    DECLARE cur_1 CURSOR FOR SELECT value, description
                             FROM synonymdomain
                             WHERE domainid = v_vObjectName
                             AND value IN ('ASSET', 'ITEM', 'LOCATIONS');

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'CLASSSTRUCTUREID' THEN v_pClassStructureID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('CLASSUSEWITHID', 'OBJECTNAME', 'DESCRIPTION', 'OBJECTVALUE')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vClassUseWithExists = 'N';

    SET v_vObjectName = 'CLASSUSEWITH';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vValue,
                              v_vDesc;

        WHILE l_sqlstatus = 0 DO

            SET v_vClassUseWithExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM classusewith
                                        WHERE classstructureid = v_pClassStructureID
                                        AND objectname = v_vValue);

            IF v_vClassUseWithExists = 'N' THEN

                SET v_vSql_stmt = 'INSERT INTO classusewith (classusewithid, objectname, description, objectvalue, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(NEXTVAL FOR CLASSUSEWITHSEQ), ' ', '') || ', ''' || v_vValue || ''', ''' || v_vDesc || ''', ''' || v_vValue || ''', ' || v_vValues || ')';

                EXECUTE IMMEDIATE v_vSql_stmt;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vValue,
                                  v_vDesc;

        END WHILE;

        CLOSE cur_1;

END 
GO

/* PROCEDURE CreateClassAncestor */
CREATE PROCEDURE PLUSG_AS_8 ( v_pClassStructureID VARCHAR(20),
                              v_pParentClassStructureID VARCHAR(20),
                              v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vClassAncestorExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vClassificationID VARCHAR(8);

    DECLARE v_iClassAncestorID INTEGER;

    DECLARE v_iCurrentHierarchyLevel INTEGER;

    DECLARE v_vParentClassificationID VARCHAR(8);

    DECLARE v_vCurrentParent VARCHAR(20);

    DECLARE v_vNextParent VARCHAR(20);

    DECLARE cur_2 CURSOR FOR SELECT classificationid
                             FROM classstructure
                             WHERE classstructureid = v_pClassStructureID;

    DECLARE cur_1 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'CLASSSTRUCTUREID' THEN v_pClassStructureID
                                    WHEN attributename = 'ANCESTOR' THEN v_pClassStructureID
                                    WHEN attributename = 'HIERARCHYLEVELS' THEN REPLACE(CHAR(v_iCurrentHierarchyLevel), ' ', '')
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('CLASSANCESTORID', 'CLASSIFICATIONID', 'ANCESTORCLASSID')
                             ORDER BY attributeno;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'CLASSSTRUCTUREID' THEN v_pClassStructureID
                                    WHEN attributename = 'CLASSIFICATIONID' THEN v_vClassificationID
                                    WHEN attributename = 'ANCESTOR' THEN v_vCurrentParent
                                    WHEN attributename = 'ANCESTORCLASSID' THEN v_vParentClassificationID
                                    WHEN attributename = 'HIERARCHYLEVELS' THEN REPLACE(CHAR(v_iCurrentHierarchyLevel), ' ', '')
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('CLASSANCESTORID')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vClassAncestorExists = 'N';

    SET v_vObjectName = 'CLASSANCESTOR';

    SET v_iCurrentHierarchyLevel = 0;

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_1;

        OPEN cur_2;

        SET l_sqlstatus = 0;
        FETCH FROM cur_2 INTO v_vClassificationID;

        WHILE l_sqlstatus = 0 DO

            SET v_vClassAncestorExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM classancestor
                                        WHERE classstructureid = v_pClassStructureID
                                        AND ancestor = v_pClassStructureID);

            SET v_iClassAncestorID = (SELECT MAX(classancestorid)
                                        FROM classancestor
                                        WHERE classstructureid = v_pClassStructureID
                                        AND ancestor = v_pClassStructureID);

            IF v_vClassAncestorExists = 'Y' THEN

		UPDATE classancestor
		SET ancestorclassid = v_vClassificationID
		WHERE classancestorid = v_iClassAncestorID;

            ELSE
		
                SET v_vSql_stmt = 'INSERT INTO classancestor (classancestorid, classificationid, ancestorclassid, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(NEXTVAL FOR CLASSANCESTORUSEQ), ' ', '') || ', ''' || v_vClassificationID || ''', ''' || v_vClassificationID || ''', ' || v_vValues || ')';

                EXECUTE IMMEDIATE v_vSql_stmt;

            END IF;

            SET v_vCurrentParent = v_pParentClassStructureID;
            SET v_iCurrentHierarchyLevel = 1;

            WHILE v_vCurrentParent IS NOT NULL DO

                SET v_vParentClassificationID = (SELECT classificationid
                                                FROM classstructure
                                                WHERE classstructureid = v_vCurrentParent);

                SET v_vNextParent = (SELECT parent
                                    FROM classstructure
                                    WHERE classstructureid = v_vCurrentParent);

                SET v_vClassAncestorExists = (SELECT COALESCE(MAX('Y'), 'N')
                                             FROM classancestor
                                             WHERE classstructureid = v_pClassStructureID
                                             AND ancestor = v_vCurrentParent);

                SET v_iClassAncestorID = (SELECT MAX(classancestorid)
                                         FROM classancestor
                                         WHERE classstructureid = v_pClassStructureID
                                         AND ancestor = v_vCurrentParent);

                IF v_vClassAncestorExists = 'Y' THEN

                    UPDATE classancestor
                        SET ancestorclassid = v_vParentClassificationID
                        WHERE classancestorid = v_iClassAncestorID;

                ELSE

        OPEN cur_0;

 	SET v_vColumns = '';
	SET v_vValues = '';

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

                        SET v_vSql_stmt = 'INSERT INTO classancestor (classancestorid, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(NEXTVAL FOR CLASSANCESTORUSEQ), ' ', '') || ', ' || v_vValues || ')';

                        EXECUTE IMMEDIATE v_vSql_stmt;

                END IF;

		SET v_iCurrentHierarchyLevel = v_iCurrentHierarchyLevel + 1;
		
		SET v_vCurrentParent = v_vNextParent;

            END WHILE;

            SET l_sqlstatus = 0;
            FETCH FROM cur_2 INTO v_vClassificationID;

        END WHILE;

        CLOSE cur_2;

END
GO

/* PROCEDURE CreateClassSpec */
CREATE PROCEDURE PLUSG_AS_9 ( v_pClassificationID VARCHAR(8),
                              v_pClassStructureID VARCHAR(20),
                              v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iClassSpecID INTEGER;

    DECLARE v_vClassSpecificationExists VARCHAR(1);

    DECLARE v_vClassSpecUseWithExists VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vAuxObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vColumns2 VARCHAR(1000);

    DECLARE v_vValues2 VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_iAssetAttributeID INTEGER;

    DECLARE v_vAssetAttrID VARCHAR(8);

    DECLARE v_vMeasureUnitID VARCHAR(8);

    DECLARE v_vDomainID VARCHAR(18);

    DECLARE v_iSequence INTEGER;

    DECLARE v_vObjectValue VARCHAR(18);

    DECLARE cur_3 CURSOR FOR SELECT value
                             FROM synonymdomain
                             WHERE domainid = 'CLASSUSEWITH'
                             AND value IN ('ASSET', 'ITEM', 'LOCATIONS');

    DECLARE cur_2 CURSOR FOR SELECT DISTINCT assetattrid, measureunitid, domainid
                             FROM plusg_as_aux4
                             WHERE classificationid = SUBSTR(v_pClassificationID,1,2) OR classificationid = SUBSTR(v_pClassificationID,1,4)
		             OR classificationid = v_pClassificationID OR SUBSTR(classificationid,1,2) = v_pClassificationID
		             OR SUBSTR(classificationid,1,4) = v_pClassificationID;

    DECLARE cur_1 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'CLASSSTRUCTUREID' THEN v_pClassStructureID
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('ASSETATTRID', 'MEASUREUNITID', 'DOMAINID', 'CLASSSPECID', 'ASSETATTRIBUTEID')
                             ORDER BY attributeno;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'CLASSSTRUCTUREID' THEN v_pClassStructureID
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vAuxObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('CLASSSPECUSEWITHID', 'CLASSSPECID', 'ASSETATTRID', 'SEQUENCE', 'OBJECTNAME', 'OBJECTVALUE')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vClassSpecificationExists = 'N';

    SET v_vClassSpecUseWithExists = 'N';

    SET v_vObjectName = 'CLASSSPEC';

    SET v_vAuxObjectName = 'CLASSSPECUSEWITH';

    SET v_iSequence = 0;

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vColumns2 = '';

    SET v_vValues2 = '';

    SET v_vSql_stmt = '';

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_1;

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns2 = '' THEN

                SET v_vColumns2 = v_vColumns2 || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues2 = v_vValues2 || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues2 = v_vValues2 || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues2 = v_vValues2 || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues2 = v_vValues2 || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues2 = v_vValues2 || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns2 = v_vColumns2 || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues2 = v_vValues2 || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues2 = v_vValues2 || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues2 = v_vValues2 || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues2 = v_vValues2 || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues2 = v_vValues2 || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_2;

        SET l_sqlstatus = 0;
        FETCH FROM cur_2 INTO v_vAssetAttrID,
                              v_vMeasureUnitID,
                              v_vDomainID;

        WHILE l_sqlstatus = 0 DO

            SET v_vClassSpecificationExists = (SELECT COALESCE(MAX('Y'), 'N')
                                                FROM classspec
                                                WHERE classstructureid = v_pClassStructureID
                                                AND assetattrid = v_vAssetAttrID
                                                AND orgid = v_pOrgID);

            IF v_vClassSpecificationExists = 'N' THEN

                SET v_iClassSpecID = NEXTVAL FOR CLASSSPECSEQ;

                SET v_iAssetAttributeID = (SELECT assetattributeid
                                           FROM assetattribute
                                           WHERE assetattrid = v_vAssetAttrID
                                           AND orgid = v_pOrgID);

                SET v_iSequence = v_iSequence + 1;
		
                SET v_vSql_stmt = 'INSERT INTO classspec (classspecid, assetattrid, measureunitid, domainid, assetattributeid, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(v_iClassSpecID), ' ', '') || ', ''' || v_vAssetAttrID || ''', ''' || v_vMeasureUnitID || ''', ''' || v_vDomainID || ''', ' || REPLACE(CHAR(v_iAssetAttributeID), ' ', '') || ', '|| v_vValues || ')';

                EXECUTE IMMEDIATE v_vSql_stmt;

                OPEN cur_3;

                SET l_sqlstatus = 0;
                FETCH FROM cur_3 INTO v_vObjectValue;

                WHILE l_sqlstatus = 0 DO

                    SET v_vClassSpecUseWithExists = (SELECT COALESCE(MAX('Y'), 'N')
                                                     FROM classspecusewith
                                                     WHERE classstructureid = v_pClassStructureID
                                                     AND assetattrid = v_vAssetAttrID
                                                     AND orgid = v_pOrgID
                                                     AND objectname = v_vObjectValue);

                    IF v_vClassSpecUseWithExists = 'N' THEN

                        SET v_vSql_stmt = 'INSERT INTO classspecusewith (classspecusewithid, classspecid, assetattrid, sequence, objectname, objectvalue, ' || v_vColumns2 || ') VALUES (' || REPLACE(CHAR(NEXTVAL FOR CLASSSPECUSEWITHSEQ), ' ', '') || ', ' || REPLACE(CHAR(v_iClassSpecID), ' ', '') || ', ''' || v_vAssetAttrID || ''', ' || REPLACE(CHAR(v_iSequence), ' ', '') || ', ''' || v_vObjectValue || ''', ''' || v_vObjectValue || ''', '|| v_vValues2 || ')';

                        EXECUTE IMMEDIATE v_vSql_stmt;

                    END IF;

                    SET l_sqlstatus = 0;
                    FETCH FROM cur_3 INTO v_vObjectValue;

                END WHILE;

                CLOSE cur_3;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_2 INTO v_vAssetAttrID,
                                  v_vMeasureUnitID,
                                  v_vDomainID;

        END WHILE;

        CLOSE cur_2;

END 
GO

/* PROCEDURE CreateClassStructure */
CREATE PROCEDURE PLUSG_AS_6 ( v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vClassStructureExists VARCHAR(1);

    DECLARE v_vClassStructureType VARCHAR(8);

    DECLARE v_vClassStructureLang VARCHAR(2);

    DECLARE v_vClassStructureID VARCHAR(20);

    DECLARE v_vLastClassStructureID VARCHAR(20);

    DECLARE v_vParentClassStructureID VARCHAR(20);

    DECLARE v_vUseClassInDesc VARCHAR(1);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vClassificationID VARCHAR(8);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE v_vParent VARCHAR(20);

    DECLARE cur_1 CURSOR FOR SELECT classificationid, description, CHAR(parent)
                             FROM plusg_as_aux6;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    WHEN attributename = 'USECLASSINDESC' THEN v_vUseClassInDesc
                                    WHEN attributename = 'TYPE' THEN v_vClassStructureType
                                    WHEN attributename = 'LANGCODE' THEN v_vClassStructureLang
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('CLASSSTRUCTUREID', 'DESCRIPTION', 'PARENT', 'CLASSIFICATIONID', 'CLASSSTRUCTUREUID', 'HASCHILDREN')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vClassStructureExists = 'N';

    SET v_vObjectName = 'CLASSSTRUCTURE';

    SET v_vClassStructureType = 'ISO14224';

    SET v_vClassStructureLang = 'EN';

    SET v_vClassStructureID = '0';

    SET v_vUseClassInDesc = '1';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vClassificationID,
                              v_vDesc,
                              v_vParent;

        WHILE l_sqlstatus = 0 DO

            IF v_vParent = '0' THEN
                SET v_vParentClassStructureID = v_vLastClassStructureID;
            ELSE
                SET v_vParentClassStructureID = null;
            END IF;

            SET v_vClassStructureExists = (SELECT COALESCE(MAX('Y'), 'N')
                                        FROM classstructure
                                        WHERE classificationid = v_vClassificationID
                                        AND COALESCE(parent, '<null>') = COALESCE(v_vParentClassStructureID, '<null>')
                                        AND orgid = v_pOrgID);

            SET v_vClassStructureID = (SELECT MAX(classstructureid)
                                       FROM classstructure
                                       WHERE classificationid = v_vClassificationID
                                       AND COALESCE(parent, '<null>') = COALESCE(v_vParentClassStructureID, '<null>')
                                       AND orgid = v_pOrgID);

            IF v_vClassStructureExists = 'N' THEN

				UPDATE autokey
				SET seed = seed + 1
				WHERE autokeyname = 'CLASSSTRUCTUREID';
		
                SET v_vClassStructureID = CHAR((SELECT (COALESCE(MAX(INTEGER(prefix)), 0) + seed)
                                           FROM autokey
                                           WHERE autokeyname = 'CLASSSTRUCTUREID'
                                           GROUP BY seed));

                IF v_vParentClassStructureID IS NOT NULL THEN
                    SET v_vSql_stmt = 'INSERT INTO classstructure (classstructureid, description, parent, classificationid, classstructureuid, haschildren, ' || v_vColumns || ') VALUES (''' || v_vClassStructureID || ''', ''' || v_vDesc || ''', ''' || v_vParentClassStructureID || ''', ''' || v_vClassificationID || ''', ' || REPLACE(CHAR(NEXTVAL FOR CLASSSTRUCTURESEQ), ' ', '') || ', 0 , ' || v_vValues || ')';
                ELSE
                    SET v_vSql_stmt = 'INSERT INTO classstructure (classstructureid, description, classificationid, classstructureuid, haschildren, ' || v_vColumns || ') VALUES (''' || v_vClassStructureID || ''', ''' || v_vDesc || ''', ''' || v_vClassificationID || ''', ' || REPLACE(CHAR(NEXTVAL FOR CLASSSTRUCTURESEQ), ' ', '') || ', 1 , ' || v_vValues || ')';
                END IF;

                EXECUTE IMMEDIATE v_vSql_stmt;

                IF v_vParent = '1' THEN

                    SET v_vLastClassStructureID = v_vClassStructureID;

                END IF;
			
			ELSE
			
                IF v_vParent = '1' THEN

                    SET v_vLastClassStructureID = v_vClassStructureID;

                END IF;

				UPDATE classstructure
                SET useclassindesc = 1
                WHERE classstructureid = v_vClassStructureID
                AND orgid = v_pOrgID;
            
            END IF;
            
            CALL PLUSG_AS_7 (v_vClassStructureID);
            CALL PLUSG_AS_8 (v_vClassStructureID, v_vParentClassStructureID, v_pOrgID);
            CALL PLUSG_AS_9 (v_vClassificationID, v_vClassStructureID, v_pOrgID);

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vClassificationID,
                                  v_vDesc,
                                  v_vParent;

        END WHILE;

        CLOSE cur_1;

END 
GO

/* Call the procedures above to define asset classification attributes */
CREATE PROCEDURE PLUSG_AS_10 (v_pOrgID VARCHAR(8) )

LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vOrgID_Exists VARCHAR(1);

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_error = '00000';

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION, SQLWARNING

        BEGIN

            SET l_error = SQLSTATE;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vOrgID_Exists = 'N';

    SELECT COALESCE(MAX('Y'), 'N')
    INTO v_vOrgID_Exists
    FROM organization
    WHERE organization.orgid = v_pOrgID
    FETCH FIRST 1 ROWS ONLY;

    IF v_vOrgID_Exists = 'Y' THEN
    	CALL PLUSG_AS_1 (v_pOrgID);
    	CALL PLUSG_AS_2 ;
    	CALL PLUSG_AS_3 ;
    	CALL PLUSG_AS_4 (v_pOrgID);
    	CALL PLUSG_AS_5 (v_pOrgID);
    	CALL PLUSG_AS_6 (v_pOrgID);    	
    END IF;

END 
GO

-- ******************************************************************************************* --
-- * End of Create Asset Catalog Hierarchies relating to BS ISO 14224...      (MXES Version) * --
-- ******************************************************************************************* --